{ **************************************************************************** }
{ *                                                                          * }
{ * MARIO BROS. (Nintendo, Game & Watch Multi Screen) Simulator              * }
{ * by Luca "MADrigal" Antignano, July 2002 (revision: September 2022)       * }
{ *                                                                          * }
{ * http://www.madrigaldesign.it/sim/                                        * }
{ * lucantignano@gmail.com                                                   * }
{ *                                                                          * }
{ * This source code (the "Software") is released as "open source" under the * }
{ * "Creative Common Attribution-NonCommercial-ShareAlike" license 4.0       * }
{ * available at:                                                            * }
{ * https://creativecommons.org/licenses/by-nc-sa/4.0/legalcode              * }
{ *                                                                          * }
{ * In addition to the constraints set by the CC license, you must respect   * }
{ * the following rules:                                                     * }
{ * a. do not offer the Software on a pay-per-play basis.                    * }
{ * b. do not distribute the Software through pre-paid access websites or    * }
{ *    BBS's.                                                                * }
{ * c. do not use the Software in publications of any form unless written    * }
{ *    consent is obtained by the author.                                    * }
{ * d. do not distribute the Software through mirror websites, unless        * }
{ *    written consent is obtained by the author.                            * }
{ * e. do not distribute older versions of the Software through authorized   * }
{ *    mirror websites after newer ones have been made available by the      * }
{ *    author.                                                               * }
{ *                                                                          * }
{ * Excerpt from the CC BY-NC-SA license 4.0                                 * }
{ * ========================================                                 * }
{ * BY: You must give appropriate credit, provide a link to the license, and * }
{ *     indicate if changes were made. You may do so in any reasonable       * }
{ *     manner, but not in any way that suggests the licensor endorses you   * }
{ *     or your use.                                                         * }
{ * NC: You may not use the material for commercial purposes.                * }
{ * SA: If you remix, transform, or build upon the material, you must        * }
{ *     distribute your contributions under the same license as the          * }
{ *     original.                                                            * }
{ *                                                                          * }
{ **************************************************************************** }

unit unit1;
{ this unit contains the main form implementation                              }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Math, Registry, Jpeg, Pngimage, Fmod, Fmodtypes, StdCtrls, Menus,
  ShellAPI;

type
  TForm1 = class(TForm)
{ * HANDHELD GRAPHICS, INCLUDING BUTTONS                                     * }

    // fullscreen mode items
    btn_game_menu_top:          TImage;
    btn_game_menu_background:   TImage;

    im_backdrop:                TImage;
    im_backdrop_logo:           TImage;

    // electronic game
    im_background_open:         TImage;     im_background_closed:       TImage; {1-2}
    im_background_screen_1_top: TImage;     im_background_screen_2_top: TImage; {3-4}

    btn_game_a_down:            TImage;     btn_game_a_top:             TImage; {5-6}
    btn_game_b_down:            TImage;     btn_game_b_top:             TImage; {7-8}
    btn_time_down:              TImage;     btn_time_top:               TImage; {9-10}
    btn_acl_down:               TImage;     btn_acl_top:                TImage; {11-12}
                                            btn_alarm_top:              TImage; {13}

    btn_left_top:               TImage;                                         {14}
    btn_left_bottom_up:         TImage;     btn_left_bottom_down:       TImage; {15-16}

    btn_right_top:              TImage;                                         {17}
    btn_right_bottom_up:        TImage;     btn_right_bottom_down:      TImage; {18-19}

    btn_ctrlr_overlay_nw:       TImage;     btn_ctrlr_overlay_ne:       TImage; {20-21}
    btn_ctrlr_overlay_sw:       TImage;     btn_ctrlr_overlay_se:       TImage; {22-23}

{ * LCD SCREEN GRAPHICS (SPRITES)                                            * }

    // on-screen panel (background, alarm, time/score, game a/b, etc.)
    im_panel_alarm_1: TImage; im_panel_alarm_2: TImage;                         {24-25}

    im_panel_game_a: TImage; im_panel_game_b: TImage;                           {26-27}

    im_panel_time_am: TImage; im_panel_time_pm: TImage; im_panel_time_colon: TImage; {28-29}

    im_panel_miss_1: TImage; im_panel_miss_2: TImage; im_panel_miss_3: TImage;  {30-32}

    im_panel_digit_pos_1: TImage; im_panel_digit_pos_2: TImage; im_panel_digit_pos_3: TImage; im_panel_digit_pos_4: TImage; {34-37}

    im_panel_number_0: TImage; im_panel_number_1: TImage; im_panel_number_2: TImage; im_panel_number_3: TImage; im_panel_number_4: TImage; im_panel_number_5: TImage; im_panel_number_6: TImage; im_panel_number_7: TImage; im_panel_number_8: TImage; im_panel_number_9: TImage; {38-47}

    // on-screen items
    im_van_1: TImage; im_van_2: TImage; im_van_3: TImage; im_van_4: TImage; im_van_5: TImage; im_van_6: TImage; {48-53}
    im_van_box_l1: TImage; im_van_box_l2: TImage; im_van_box_l3: TImage; im_van_box_l4: TImage; im_van_box_l5: TImage; {54-58}
    im_van_box_r1: TImage; im_van_box_r2: TImage; im_van_box_r3: TImage; im_van_box_r4: TImage; im_van_box_r5: TImage; {59-63}

    im_hero_l_3_1: TImage; im_hero_l_3_2: TImage; im_hero_l_3_3: TImage;
    im_hero_l_2_1: TImage; im_hero_l_2_2: TImage; im_hero_l_2_3: TImage;
    im_hero_l_1_1: TImage; im_hero_l_1_2: TImage; im_hero_l_1_3: TImage;

    im_hero_r_3_1: TImage; im_hero_r_3_2: TImage; im_hero_r_3_3: TImage;
    im_hero_r_2_1: TImage; im_hero_r_2_2: TImage; im_hero_r_2_3: TImage;
    im_hero_r_1_1: TImage; im_hero_r_1_2: TImage; im_hero_r_1_3: TImage;

    im_bonus_l_1: TImage; im_bonus_l_2: TImage;
    im_bonus_r_1: TImage; im_bonus_r_2: TImage;

    im_miss_l_1: TImage; im_miss_l_2: TImage; im_miss_l_3: TImage; im_miss_l_4: TImage;
    im_miss_r_1: TImage; im_miss_r_2: TImage; im_miss_r_3: TImage; im_miss_r_4: TImage; im_miss_r_5: TImage;

                                                                                   im_box_39_in: TImage;                                    im_box_36_b: TImage;
    im_box_43: TImage; im_box_42: TImage; im_box_41: TImage; im_box_40: TImage; {} im_box_39: TImage; im_box_38: TImage; im_box_37: TImage; im_box_36: TImage;

    im_box_28_b: TImage;                                     im_box_31_in: TImage;
    im_box_28: TImage; im_box_29: TImage; im_box_30: TImage; im_box_31: TImage; {} im_box_32: TImage; im_box_33: TImage; im_box_34: TImage; im_box_35: TImage;

                                                                                   im_box_23_in: TImage;                                    im_box_20_b: TImage;
    im_box_27: TImage; im_box_26: TImage; im_box_25: TImage; im_box_24: TImage; {} im_box_23: TImage; im_box_22: TImage; im_box_21: TImage; im_box_20: TImage;

    im_box_12_b: TImage;                                     im_box_15_in: TImage;
    im_box_12: TImage; im_box_13: TImage; im_box_14: TImage; im_box_15: TImage; {} im_box_16: TImage; im_box_17: TImage; im_box_18: TImage; im_box_19: TImage;

                                                                                   im_box_07_in: TImage;                                    im_box_04_b: TImage;
    im_box_11: TImage; im_box_10: TImage; im_box_09: TImage; im_box_08: TImage; {} im_box_07: TImage; im_box_06: TImage; im_box_05: TImage; im_box_04: TImage; {} im_box_03: TImage; im_box_02: TImage; im_box_01: TImage;

{ * TIMERS                                                                   * }
    Timer_Bonus:            TTimer;
    Timer_Box_To_Van:       TTimer;
    Timer_Chance:           TTimer;
    Timer_Clock:            TTimer;
    Timer_Game:             TTimer;
    Timer_Game_Over:        TTimer;
    Timer_Luigi_Moved_Box:  TTimer;
    Timer_Luigi_Moving_Box: TTimer;
    Timer_Mario_Moved_Box:  TTimer;
    Timer_Mario_Moving_Box: TTimer;
    Timer_Miss:             TTimer;
    Timer_Move_Luigi_Up:    TTimer;
    Timer_Move_Luigi_Down:  TTimer;
    Timer_Move_Mario_Up:    TTimer;
    Timer_Move_Mario_Down:  TTimer;
    Timer_Score_Blink:      TTimer;
    Timer_Van_Smoke:        TTimer;

{ * POPUP MENU                                                               * }
    pm_0_menu:          TPopupMenu;
    pm_1_about:         TMenuItem;
    pm_2_readme:        TMenuItem;
    pm_3_whatsnew:      TMenuItem;
    pm_4_manual:        TMenuItem;
    pm_5_n1:            TMenuItem;
    pm_6_config:        TMenuItem;
    pm_7_keys:          TMenuItem;
    pm_8_hiscore:       TMenuItem;
    pm_9_n2:            TMenuItem;
  {}pm_10_fullscreen:   TMenuItem;
  {}pm_11_splashscreen: TMenuItem;
  {}pm_12_controller:   TMenuItem;
  {}pm_13_n3:           TMenuItem;
  {}pm_14_exit:         TMenuItem;

{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
    procedure FormCreate                    (Sender: TObject);
    procedure FormActivate                  (Sender: TObject);
    procedure FormClose                     (Sender: TObject; var Action: TCloseAction);

{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
    procedure Vid_Display_Ctrl_Overlay;

    procedure Vid_Show_All                  (b: boolean);

    procedure Vid_Show_All_Luigi            (b: boolean);
    procedure Vid_Show_All_Mario            (b: boolean);
    procedure Vid_Show_All_Boxes_In_Game    (b: boolean);
    procedure Vid_Show_All_Boxes_On_Van     (b: boolean);
    procedure Vid_Show_All_Van              (b: boolean);
    procedure Vid_Show_All_Bonus            (b: boolean);
    procedure Vid_Show_All_Miss             (b: boolean);

    procedure Vid_Show_Panel_Miss           (var p: integer);
    procedure Vid_Show_Panel_Score          (var p: integer; b: boolean);

    procedure Vid_Show_Luigi                (var p: integer; b: boolean);
    procedure Vid_Show_Mario                (var p: integer; b: boolean);

{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
    procedure Snd_Play                      (pfs: PFSoundSample);
    procedure Snd_Silence;

{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
    procedure im_background_closedClick     (Sender: TObject);

    procedure im_background_openMouseDown   (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure im_background_openMouseMove   (Sender: TObject;                       Shift: TShiftState; X, Y: Integer);

    procedure btn_game_a_topMouseDown       (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_a_topMouseUp         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_b_topMouseDown       (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_b_topMouseUp         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_time_topMouseDown         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_time_topMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_acl_topMouseDown          (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_acl_topMouseUp            (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure btn_ctrlr_overlay_nwMouseDown (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_nwMouseUp   (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_swMouseDown (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_swMouseUp   (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_neMouseDown (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_neMouseUp   (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_seMouseDown (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_seMouseUp   (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure FormKeyDown                   (Sender: TObject; var Key: Word;        Shift: TShiftState);
    procedure FormKeyUp                     (Sender: TObject; var Key: Word;        Shift: TShiftState);

{ * INPUT MANAGEMENT - ACTIONS                                               * }
    procedure Inp_game_a_down;              procedure Inp_game_a_up;
    procedure Inp_game_b_down;              procedure Inp_game_b_up;
    procedure Inp_time_down;                procedure Inp_time_up;
    procedure Inp_acl_down;                 procedure Inp_acl_up;

    procedure Inp_1_nw_down;                procedure Inp_1_nw_up;
    procedure Inp_2_sw_down;                procedure Inp_2_sw_up;
    procedure Inp_3_ne_down;                procedure Inp_3_ne_up;
    procedure Inp_4_se_down;                procedure Inp_4_se_up;

{ * PROCEDURES: GAME ROUTINES                                                * }
    procedure Gam_Stop_All_Timers;

    procedure Gam_Check_Collision_Luigi;
    procedure Gam_Check_Collision_Mario;
    procedure Gam_Move_Luigi_Down;
    procedure Gam_Move_Luigi_Up;
    procedure Gam_Move_Mario_Down;
    procedure Gam_Move_Mario_Up;
    procedure Gam_Set_Max_Boxes_On_Screen;
    procedure Gam_Set_Mode;
    procedure Gam_Set_Timer_Game_Interval;
    procedure Gam_Start_Chance_Time;

{ * PROCEDURES: TIMERS                                                       * }
    procedure Timer_BonusTimer              (Sender: TObject);
    procedure Timer_Box_To_VanTimer         (Sender: TObject);
    procedure Timer_ChanceTimer             (Sender: TObject);
    procedure Timer_ClockTimer              (Sender: TObject);
    procedure Timer_GameTimer               (Sender: TObject);
    procedure Timer_Game_OverTimer          (Sender: TObject);
    procedure Timer_Luigi_Moved_BoxTimer    (Sender: TObject);
    procedure Timer_Luigi_Moving_BoxTimer   (Sender: TObject);
    procedure Timer_Mario_Moved_BoxTimer    (Sender: TObject);
    procedure Timer_Mario_Moving_BoxTimer   (Sender: TObject);
    procedure Timer_MissTimer               (Sender: TObject);
    procedure Timer_Move_Luigi_DownTimer    (Sender: TObject);
    procedure Timer_Move_Luigi_UpTimer      (Sender: TObject);
    procedure Timer_Move_Mario_DownTimer    (Sender: TObject);
    procedure Timer_Move_Mario_UpTimer      (Sender: TObject);
    procedure Timer_Score_BlinkTimer        (Sender: TObject);
    procedure Timer_Van_SmokeTimer          (Sender: TObject);

{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
    procedure pm_0_menuPopup                (Sender: TObject);
    procedure pm_1_aboutClick               (Sender: TObject);
    procedure pm_2_readmeClick              (Sender: TObject);
    procedure pm_3_whatsnewClick            (Sender: TObject);
    procedure pm_4_manualClick              (Sender: TObject);
    procedure pm_6_configClick              (Sender: TObject);
    procedure pm_7_keysClick                (Sender: TObject);
    procedure pm_8_hiscoreClick             (Sender: TObject);
  {}procedure pm_10_fullscreenClick         (Sender: TObject);
  {}procedure pm_11_splashscreenClick       (Sender: TObject);
  {}procedure pm_12_controllerClick         (Sender: TObject);
  {}procedure pm_14_exitClick               (Sender: TObject);

{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
    procedure btn_game_menu_topClick        (Sender: TObject);

    procedure la_menu_itemMouseEnter        (Sender: TObject);
    procedure la_menu_itemMouseLeave        (Sender: TObject);
    procedure la_menu_itemMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure Vid_Display_Game_Menu         (b: boolean);
  end;

const
{ introduced for use with HH2 / now incorporated into main source }

  HH2_CONTENT_NAME    = 'Mario Bros. (Nintendo, Game & Watch Multi Screen)';
  HH2_CONTENT_AUTHOR  = 'Luca "MADrigal" Antignano';
  HH2_CONTENT_EMAIL   = 'lucantignano@gmail.com';
  HH2_CONTENT_VERSION = '5.01';
  HH2_CONTENT_DATE    = '2022/09/26';
  HH2_CONTENT_WEBSITE = 'www.madrigaldesign.it/sim';

  { constants for all builds }
  
  APP_NAME     = HH2_CONTENT_NAME + ' Simulator';                               // application's name (appears on taskbar and Splash screen)
{}APP_VERSION  = 'Version ' + HH2_CONTENT_VERSION + ' * Graphic improvements + minor fixes *';// current version (appears on Splash screen)
{}APP_DATE     = 'Released by ' + HH2_CONTENT_AUTHOR + ' on 26th September, 2022'; // release date (appears on Splash screen)

  URL_EMAIL    = 'mailto:' + HH2_CONTENT_EMAIL;                                 // link to the email address
  URL_PAYPAL   = 'https://www.paypal.me/LucaAntignano';                         // link to the Paypal donation page
  URL_WEBSITE  = 'http://' + HH2_CONTENT_WEBSITE + '/';                         // link to the website

  REG_KEY      =  'Software\MADrigal\MarioBro50';                               // registry key

  HINT_CLICK   = 'click or press';
  HINT_OPEN    = 'to open the game';
  HINT_GAME_A  = 'to start game A (hold to view highscore)';
  HINT_GAME_B  = 'to start game B (hold to view highscore)';
  HINT_TIME    = 'to view current time (hold to view alarm time)';
  HINT_ACL     = 'to reset the game';

  HINT_ALARM   = 'button disabled';
  HINT_BTNMOV  = 'activate the controller overlays or use';

{}MENU_ITEMS = 14;

var
{ * main objects * }
  Form1:                   TForm1;                                              // main form
  bStartup:                boolean;                                             // manages splash screen at startup
  iMouseX, iMouseY:        integer;                                             // cursor coordinates (to drag main form)
  iReg:                    TRegIniFile;                                         // system registry (to save high-scores)

{ * sound management * }
  pfs_movebox,
  pfs_movechar,
  pfs_miss1,
  pfs_miss2,
  pfs_point:               PFSoundSample;                                       // sound samples

  bSound:                  boolean;                                             // sound enabled? (depending on selected driver on splash screen)

{ * arrays of images * }
  aIm_panel_digit_pos:     array[1..4]        of TImage;                        // empty sprites to host score/time digits
  aIm_panel_alarm:         array[1..2]        of TImage;                        // sprites for alarm (unused)
  aIm_panel_miss:          array[1..3]        of TImage;                        // sprites for misses panel
  aIm_panel_number:        array[0..9]        of TImage;                        // digits for score/time

  aIm_hero_l,                                                                   // Luigi, working (i = row; j = body/hands)
  aIm_hero_r:              array[1..3,  1..3] of TImage;                        // Mario, working (i = row; j = body/hands)

  aIm_box:                 array[0..12, 1..5] of TImage;                        // boxes (x=0 and x=12 needed to get correct timing)

  aIm_van:                 array[1..6]        of TImage;                        // sprites for van, driver and smoke
  aIm_van_box:             array[1..2,  1..5] of TImage;                        // sprites for boxes on the van

  aIm_bonus:               array[1..2,  1..2] of TImage;                        // Luigi (1,n) and Mario (2,n) in bonus intermission
  aIm_miss:                array[1..2,  1..5] of TImage;                        // Luigi (1,n) and Mario (2,n) in miss intermission

  aIm_Miss_Code:           array[1..6]        of TImage;                        // which box is gonna break in "timer_miss" routine

{ * arrays of mixed types *}
  aKeys_int:               array[1..4]        of integer;
  aKeys_word:              array[1..4]        of word;
  aMaxBoxes:               array[6..8, 0..9, 0..4] of integer;

  aKeys:                   array[1..9]       of record                          // keys management (values)
                                                  int:  integer;
                                                  word: word;
                                                end;

  aKeyHints:               array[1..40] of string;                              // keys management (hint on buttons)

  aMenuItem:               array[1..MENU_ITEMS] of TMenuItem;                   // popup menu items
  aMenuLabels:             array[1..MENU_ITEMS] of TLabel;                      // on-screen game menu labels

{ * integers and booleans * }
  iMode,                                                                        // 0=closed, 1/2=ACL 1/2, 3=alarm, 4=time, 5/7=hiscore A/B, 6/8=game A/B, 9=game over
  iScore,                                                                       // score=0-999
  iHiScoreA,                                                                    // hiscore for Game A=0-999
  iHiScoreB,                                                                    // hiscore for Game B=0-999
  iScoreMultiplier,                                                             // given points=1 (normal) or 2 (chance-time)

  iPosLuigi,                                                                    // Y coordinate of Luigi
  iPosMario,                                                                    // Y coordinate of Mario
  iMiss,                                                                        // number of misses: 0-3
  iBoxesOnVan,                                                                  // number of boxes on the van: 0-8
  iMaxBoxesOnScreen                                                             // maximum amount of boxes on screen: 0-9
                         : integer;

{}bFullscreenMode,                                                              // true if video mode is fullscreen (false if "shaped game")
  bGameMenuOpen,                                                                // true if the on-screen menu is visible
  bDisplayCtrlOverlays,                                                         // true if the controller overlay should be displayed

  bUpPress,
  bDownPress,
  bLeftPress,
  bRightPress,

  bNWKeyPressed,
  bNEKeyPressed,
  bSWKeyPressed,
  bSEKeyPressed,

  bControllerLuigi,
  bControllerMario
                         : boolean;

  // Game variables: loops
  iLoopBonus,
  iLoopBoxToVan,
  iLoopChance,
  iLoopGame,
  iLoopGameOver,
  iLoopMiss,
  iLoopMissCode,
  iLoopMovingBoxCode,
  iLoopScoreBlink
                         : integer;

implementation

uses
  about, gfxinit, keys, manual, splash;

{$r *.dfm}










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.FormCreate(Sender: TObject);
{ sprites on form are associated to arrays of images, needed to "index" them   }
{ variables are initialized: system registry, randomize, game mode, etc.       }
  var
    i, j: integer;
begin
  // initializes application name (for Windows taskbar)
  Application.Title := APP_NAME;

{}// sets game off the screen while graphics initialise
{}Top  := - Screen.Height;
{}Left := - Screen.Width;

  // on-screen panel
  aIm_panel_alarm[1] := im_panel_alarm_1; aIm_panel_alarm[2] := im_panel_alarm_2;
  aIm_panel_miss[1] := im_panel_miss_1; aIm_panel_miss[2] := im_panel_miss_2; aIm_panel_miss[3] := im_panel_miss_3;
  aIm_panel_digit_pos[1] := im_panel_digit_pos_1;  aIm_panel_digit_pos[2] := im_panel_digit_pos_2; aIm_panel_digit_pos[3] := im_panel_digit_pos_3; aIm_panel_digit_pos[4] := im_panel_digit_pos_4;

  // numbers
  aIm_panel_number[0] := im_panel_number_0; aIm_panel_number[1] := im_panel_number_1; aIm_panel_number[2] := im_panel_number_2; aIm_panel_number[3] := im_panel_number_3; aIm_panel_number[4] := im_panel_number_4;
  aIm_panel_number[5] := im_panel_number_5; aIm_panel_number[6] := im_panel_number_6; aIm_panel_number[7] := im_panel_number_7; aIm_panel_number[8] := im_panel_number_8; aIm_panel_number[9] := im_panel_number_9;

  // on-screen items
  aIm_hero_l[3, 1] := im_hero_l_3_1; aIm_hero_l[3, 2] := im_hero_l_3_2; aIm_hero_l[3, 3] := im_hero_l_3_3;
  aIm_hero_l[2, 1] := im_hero_l_2_1; aIm_hero_l[2, 2] := im_hero_l_2_2; aIm_hero_l[2, 3] := im_hero_l_2_3;
  aIm_hero_l[1, 1] := im_hero_l_1_1; aIm_hero_l[1, 2] := im_hero_l_1_2; aIm_hero_l[1, 3] := im_hero_l_1_3;

  aIm_hero_r[3, 1] := im_hero_r_3_1; aIm_hero_r[3, 2] := im_hero_r_3_2; aIm_hero_r[3, 3] := im_hero_r_3_3;
  aIm_hero_r[2, 1] := im_hero_r_2_1; aIm_hero_r[2, 2] := im_hero_r_2_2; aIm_hero_r[2, 3] := im_hero_r_2_3;
  aIm_hero_r[1, 1] := im_hero_r_1_1; aIm_hero_r[1, 2] := im_hero_r_1_2; aIm_hero_r[1, 3] := im_hero_r_1_3;

  aIm_miss[1, 1] := im_miss_l_1; aIm_miss[1, 2] := im_miss_l_2; aIm_miss[1, 3] := im_miss_l_3; aIm_miss[1, 4] := im_miss_l_4;
  aIm_miss[2, 1] := im_miss_r_1; aIm_miss[2, 2] := im_miss_r_2; aIm_miss[2, 3] := im_miss_r_3; aIm_miss[2, 4] := im_miss_r_4; aIm_miss[2, 5] := im_miss_r_5;

  aIm_van[1] := im_van_1; aIm_van[2] := im_van_2; aIm_van[3] := im_van_3; aIm_van[4] := im_van_4; aIm_van[5] := im_van_5; aIm_van[6] := im_van_6;

  aIm_van_box[1, 1] := im_van_box_l1; aIm_van_box[2, 1] := im_van_box_r1;
  aIm_van_box[1, 2] := im_van_box_l2; aIm_van_box[2, 2] := im_van_box_r2;
  aIm_van_box[1, 3] := im_van_box_l3; aIm_van_box[2, 3] := im_van_box_r3;
  aIm_van_box[1, 4] := im_van_box_l4; aIm_van_box[2, 4] := im_van_box_r4;
  aIm_van_box[1, 5] := im_van_box_l5; aIm_van_box[2, 5] := im_van_box_r5;

  aIm_bonus[1, 1] := im_bonus_l_1; aIm_bonus[2, 1] := im_bonus_r_1;
  aIm_bonus[1, 2] := im_bonus_l_2; aIm_bonus[2, 2] := im_bonus_r_2;

  { boxes, left screen }                                                                                          { boxes, right screen }
                                                                                                                     aIm_box[4, 5] := im_box_39_in;                                                      aIm_box[8, 5] := im_box_36_b;
  aIm_box[0, 5] := im_box_43; aIm_box[1, 5] := im_box_42; aIm_box[2, 5] := im_box_41; aIm_box[3, 5] := im_box_40; {} aIm_box[5, 5] := im_box_39; aIm_box[6, 5] := im_box_38; aIm_box[7, 5] := im_box_37; aIm_box[9, 5] := im_box_36;

  aIm_box[1, 4] := im_box_28_b;                                                       aIm_box[5, 4] := im_box_31_in;
  aIm_box[0, 4] := im_box_28; aIm_box[2, 4] := im_box_29; aIm_box[3, 4] := im_box_30; aIm_box[4, 4] := im_box_31; {} aIm_box[6, 4] := im_box_32; aIm_box[7, 4] := im_box_33; aIm_box[8, 4] := im_box_34; aIm_box[9, 4] := im_box_35;

                                                                                                                     aIm_box[4, 3] := im_box_23_in;                                                      aIm_box[8, 3] := im_box_20_b;
  aIm_box[0, 3] := im_box_27; aIm_box[1, 3] := im_box_26; aIm_box[2, 3] := im_box_25; aIm_box[3, 3] := im_box_24; {} aIm_box[5, 3] := im_box_23; aIm_box[6, 3] := im_box_22; aIm_box[7, 3] := im_box_21; aIm_box[9, 3] := im_box_20;

  aIm_box[1, 2] := im_box_12_b;                                                       aIm_box[5, 2] := im_box_15_in;
  aIm_box[0, 2] := im_box_12; aIm_box[2, 2] := im_box_13; aIm_box[3, 2] := im_box_14; aIm_box[4, 2] := im_box_15; {} aIm_box[6, 2] := im_box_16; aIm_box[7, 2] := im_box_17; aIm_box[8, 2] := im_box_18; aIm_box[9, 2] := im_box_19;

                                                                                                                     aIm_box[4, 1] := im_box_07_in;                                                      aIm_box[8, 1] := im_box_04_b;
  aIm_box[0, 1] := im_box_11; aIm_box[1, 1] := im_box_10; aIm_box[2, 1] := im_box_09; aIm_box[3, 1] := im_box_08; {} aIm_box[5, 1] := im_box_07; aIm_box[6, 1] := im_box_06; aIm_box[7, 1] := im_box_05; aIm_box[9, 1] := im_box_04; {} aIm_box[10, 1] := im_box_03; aIm_box[11, 1] := im_box_02; aIm_box[12, 1] := im_box_01;

  // inits "aIm_Miss_Code", which is used in Timer_Miss routine
  aIm_miss_code[6] := aIm_box[0, 5]; { Luigi top    }     aIm_miss_code[5] := aIm_box[ 9, 4]; { Mario top    }
  aIm_miss_code[4] := aIm_box[0, 3]; { Luigi middle }     aIm_miss_code[3] := aIm_box[ 9, 2]; { Mario middle }
  aIm_miss_code[2] := aIm_box[0, 1]; { Luigi bottom }     aIm_miss_code[1] := aIm_box[10, 1]; { Mario bottom }

  // inits "aMaxBoxes, which is used to set maximum possible boxes on screen
  for i := 0 to 4 do for j := 0 to 4 do aMaxBoxes[6, i * 2,     j]     := (j div 2) + 2 + i;
  for i := 1 to 5 do for j := 1 to 5 do aMaxBoxes[6, i * 2 - 1, j - 1] := (j div 2) + 1 + i;
  for i := 0 to 9 do for j := 1 to 5 do aMaxBoxes[8, i,         j - 1] := aMaxBoxes[6, i, j] + 1;
  aMaxBoxes[6, 0, 1] := 3; // forces value to 3
  aMaxBoxes[8, 1, 4] := 6; // forces value to 6

  // registry keys management
  iReg := TregIniFile.Create(REG_KEY);                                          // system registry init

{}// init graphics
{}bFullscreenMode := iReg.ReadBool('', 'FullscreenMode', true);
{}pm_10_fullscreen.Checked := bFullscreenMode;
{}Gfxinit.Gfx_Game_Init;
{}bGameMenuOpen := false;

{}// initialises the "hints" array
  for i := 1 to 10 do
    aKeyHints[i] := '''' + Chr(i + 47) + '''';

  for i := 11 to 36 do
    aKeyHints[i] := '''' + Chr(i + 54) + '''';

  aKeyHints[37] := '''alt'' key';
  aKeyHints[38] := '''ctrl'' key';
  aKeyHints[39] := '''shift'' key';
  aKeyHints[40] := '''spacebar''';

  // reads values from system registry into the inputs array
  aKeys[1].int := iReg.ReadInteger('', 'Key1', 2);                              // "Game A"          (default: "1")
  aKeys[2].int := iReg.ReadInteger('', 'Key2', 3);                              // "Game B"          (default: "2")
  aKeys[3].int := iReg.ReadInteger('', 'Key3', 4);                              // "Time"            (default: "3")
  aKeys[4].int := iReg.ReadInteger('', 'Key4', 6);                              // "ACL"             (default: "5")
  aKeys[5].int := iReg.ReadInteger('', 'Key5', 27);                             // "NW (Luigi up)"   (default: "Q")
  aKeys[6].int := iReg.ReadInteger('', 'Key6', 11);                             // "SW (Luigi down)" (default: "A")
  aKeys[7].int := iReg.ReadInteger('', 'Key7', 26);                             // "NE (Mario up)"   (default: "P")
  aKeys[8].int := iReg.ReadInteger('', 'Key8', 22);                             // "SE (Mario down)" (default: "L")

  // writes values into the system registry (in case no pre-existing values
  // stored), then assigns key values to the key-inputs array
  for i := 1 to 8 do
    begin
      iReg.WriteInteger('', 'Key' + inttostr(i), aKeys[i].int);
      KeysForm.AssignWord(i, aKeys[i].int);
    end;

  // sets button images hints
  im_background_closed.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[4].int] + ' ' + HINT_OPEN;    // "Open"

  btn_game_a_top.Hint       := HINT_CLICK  + ' ' + aKeyHints[aKeys[1].int] + ' ' + HINT_GAME_A; // "Game A"
  btn_game_b_top.Hint       := HINT_CLICK  + ' ' + aKeyHints[aKeys[2].int] + ' ' + HINT_GAME_B; // "Game B"
  btn_time_top.Hint         := HINT_CLICK  + ' ' + aKeyHints[aKeys[3].int] + ' ' + HINT_TIME;   // "Time"
  btn_acl_top.Hint          := HINT_CLICK  + ' ' + aKeyHints[aKeys[4].int] + ' ' + HINT_ACL;    // "ACL"
  btn_alarm_top.Hint        := HINT_ALARM;                                                      // "Alarm"

  btn_left_top.Hint         := HINT_BTNMOV + ' ' + aKeyHints[aKeys[5].int] + ', '               // "NW (Luigi up)"
                                                 + aKeyHints[aKeys[6].int];                     // "SW (Luigi down)"
  btn_right_top.Hint        := HINT_BTNMOV + ' ' + aKeyHints[aKeys[7].int] + ', '               // "NE (Mario up)"
                                                 + aKeyHints[aKeys[8].int];                     // "SE (Mario down)"

  btn_ctrlr_overlay_nw.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[5].int];                      // "NW (Luigi up)"
  btn_ctrlr_overlay_sw.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[6].int];                      // "SW (Luigi down)"
  btn_ctrlr_overlay_ne.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[7].int];                      // "NE (Mario up)"
  btn_ctrlr_overlay_se.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[8].int];                      // "SE (Mario down)"

  // popup menu array association
  aMenuItem[1]  := pm_1_about;
  aMenuItem[2]  := pm_2_readme;
  aMenuItem[3]  := pm_3_whatsnew;
  aMenuItem[4]  := pm_4_manual;
  aMenuItem[5]  := pm_5_n1;
  aMenuItem[6]  := pm_6_config;
  aMenuItem[7]  := pm_7_keys;
  aMenuItem[8]  := pm_8_hiscore;
  aMenuItem[9]  := pm_9_n2;
{}aMenuItem[10] := pm_10_fullscreen;
{}aMenuItem[11] := pm_11_splashscreen;
{}aMenuItem[12] := pm_12_controller;
{}aMenuItem[13] := pm_13_n3;
{}aMenuItem[14] := pm_14_exit;

  // controller overlay setting
{}pm_12_controller.Checked  := iReg.ReadBool('', 'CtrlrOverlay', true);
{}bDisplayCtrlOverlays := pm_12_controller.Checked;

  // variables init
  DoubleBuffered := true;                                                       // enables double buffering rendering

  iHiScoreA := iReg.ReadInteger('', 'HiScore1', iHiScoreA);                     // reads value from system registry into iHiScoreA
  iHiScoreB := iReg.ReadInteger('', 'HiScore2', iHiScoreB);                     // reads value from system registry into iHiScoreB

  bControllerLuigi := false;                                                    // Luigi can't move
  bControllerMario := false;                                                    // Mario can't move

  bUpPress    := false;
  bDownPress  := false;
  bLeftPress  := false;
  bRightPress := false;

  bNWKeyPressed := false;
  bNEKeyPressed := false;
  bSWKeyPressed := false;
  bSEKeyPressed := false;

  Randomize;                                                                    // initializes the random number generator
end;





procedure TForm1.FormActivate(Sender: TObject);
{ sends focus to the splash screen at startup                                  }
begin
  if bStartup then
    begin
      bStartup := false;
      SplashForm.Show;
    end;
end;





procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
{ frees memory before shutting down                                            }
begin
  FSOUND_Sample_Free(pfs_miss1);   FSOUND_Sample_Free(pfs_miss2);
  FSOUND_Sample_Free(pfs_movebox); FSOUND_Sample_Free(pfs_movechar);
  FSOUND_Sample_Free(pfs_point);

  FSOUND_Close;

{}iReg.Destroy;
  Action := caFree;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Vid_Display_Ctrl_Overlay;
{ displays or hides the controller overlay images                              }
begin
  btn_ctrlr_overlay_nw.Visible := bDisplayCtrlOverlays and im_background_open.Visible;
  btn_ctrlr_overlay_sw.Visible := bDisplayCtrlOverlays and im_background_open.Visible;
  btn_ctrlr_overlay_ne.Visible := bDisplayCtrlOverlays and im_background_open.Visible;
  btn_ctrlr_overlay_se.Visible := bDisplayCtrlOverlays and im_background_open.Visible;
end;





procedure TForm1.Vid_Show_All(b: boolean);
{ displays/hides all sprites                                                   }
  var
    i: integer;
begin
  for i := 1 to 4 do
    aIm_panel_digit_pos[i].Visible := true;

  if b then
    begin
      aIm_panel_digit_pos[1].Picture := aIm_panel_number[1].Picture;
      aIm_panel_digit_pos[2].Picture := aIm_panel_number[2].Picture;
      aIm_panel_digit_pos[3].Picture := aIm_panel_number[0].Picture;
      aIm_panel_digit_pos[4].Picture := aIm_panel_number[0].Picture;
    end
  else
    for i := 1 to 4 do
      aIm_panel_digit_pos[1].Picture := nil;

  im_panel_game_a.Visible       := b;
  im_panel_game_b.Visible       := b;

  im_panel_time_am.Visible      := b;
  im_panel_time_pm.Visible      := false;
  im_panel_time_colon.Visible   := b;

  for i := 1 to 2 do
    aIm_panel_alarm[i].Visible  := b;

  i := Ord(b) * 3; Vid_Show_Panel_Miss(i);

  Vid_Show_All_Luigi(b);
  Vid_Show_All_Mario(b);
  Vid_Show_All_Boxes_In_Game(b);
  Vid_Show_All_Boxes_On_Van(b);
  Vid_Show_All_Van(b);
  Vid_Show_All_Bonus(b);
  Vid_Show_All_Miss(b);
end;





procedure TForm1.Vid_Show_All_Luigi(b: boolean);
{ displays/hides all Luigi's sprites at once                                   }
  var
    i, j: integer;
begin
  for i := 1 to 3 do
    for j := 1 to 3 do
      aIm_hero_l[i, j].Visible := b;
end;





procedure TForm1.Vid_Show_All_Mario(b: boolean);
{ displays/hides all Mario's sprites at once                                   }
  var
    i, j: integer;
begin
  for i := 1 to 3 do
    for j := 1 to 3 do
      aIm_hero_r[i, j].Visible := b;
end;





procedure TForm1.Vid_Show_All_Bonus(b: boolean);
{ displays/hides all the sprites of the "bonus" intermission                   }
  var
   i, j: integer;
begin
  for i := 1 to 2 do
    for j := 1 to 2 do
      aIm_bonus[i, j].Visible := b;
end;





procedure TForm1.Vid_Show_All_Boxes_In_Game(b: boolean);
{ displays/hides all boxes on the conveyor belts at once                       }
  var
    i, j :integer;
begin
  for i := 0 to 9 do
    for j := 1 to 5 do aIm_box[i, j].Visible := b;

  for i := 10 to 12 do
    aIm_box[i, 1].Visible := b;
end;





procedure TForm1.Vid_Show_All_Boxes_On_Van(b: boolean);
{ displays/hides all boxes on the van at once                                  }
  var
    i, j: integer;
begin
  for i := 1 to 2 do
    for j :=1 to 5 do
      aIm_van_box[i, j].Visible := b;
end;





procedure TForm1.Vid_Show_All_Miss(b: boolean);
{ displays/hides all the sprites of the "miss" intermission                    }
  var
    i: integer;
begin
  for i := 1 to 4 do
    aIm_miss[1, i].Visible := b;

  for i := 1 to 5 do
    aIm_miss[2, i].Visible := b;
end;





procedure TForm1.Vid_Show_All_Van(b: boolean);
{ displays/hides boxes all the pieces the van is made of                       }
  var
    i: integer;
begin
  for i := 1 to 6 do
    aIm_van[i].Visible := b;
end;





procedure TForm1.Vid_Show_Panel_Miss(var p: integer);
{ displays misses panel in various configurations                              }
  var
    i: integer;
begin
  for i := 1 to 3 do
    aIm_panel_miss[i].Visible := p >= i;
end;





procedure TForm1.Vid_Show_Panel_Score(var p: integer; b: boolean);
{ transforms score/high-score into sprites, and displays/hides them            }
  var
    i, t: integer;
begin
  t := p;

  while t >= 1000 do                                                            // displays "1000" as "...0"
    Dec(t, 1000);

  aIm_panel_digit_pos[1].Picture := nil;                                        // thousands (always invisible)

  if b then                                                                     // displays or hide score/high-score?
    begin                                                                               // displays
      if t > 99 then aIm_panel_digit_pos[2].Picture := aIm_panel_number[t div 100 mod 10].picture else aIm_panel_digit_pos[2].Picture := nil; // hundreds
      if t >  9 then aIm_panel_digit_pos[3].Picture := aIm_panel_number[t div  10 mod 10].picture else aIm_panel_digit_pos[3].Picture := nil; // tens
                     aIm_panel_digit_pos[4].Picture := aIm_panel_number[t         mod 10].picture;                                            // units
    end
  else                                                                                  // hides
    for i := 2 to 4 do
      aIm_panel_digit_pos[i].picture := nil;
end;





procedure TForm1.Vid_Show_Luigi(var p: integer; b: boolean);
{ displays/hides Luigi in "p" position                                         }
  var
    i: integer;
begin
  for i := 1 to 2 do
    aIm_hero_l[p, i].Visible := b;

  aIm_hero_l[p, 3].Visible := false;
end;





procedure TForm1.Vid_Show_Mario(var p: integer; b: boolean);
{ displays/hides Mario in "p" position                                         }
  var
    i: integer;
begin
  for i := 1 to 2 do
    aIm_hero_r[p, i].Visible := b;

  aIm_hero_r[p, 3].Visible := false;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Snd_Play(pfs: PFSoundSample);
{ plays a certain sound sample                                                 }
begin
  if bSound then
    FSOUND_PlaySound(FSOUND_FREE, pfs);
end;





procedure TForm1.Snd_Silence;
{ stops any currently playing sound sample                                     }
begin
  if bSound then
    FSOUND_StopSound(FSOUND_ALL);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.im_background_closedClick(Sender: TObject);
{ opening the handheld game                                                    }
begin
  // background
  im_background_closed.Visible := false;
  im_background_open.Visible   := true;

  im_background_screen_1_top.Visible := true;
  im_background_screen_2_top.Visible := true;

  // buttons
  btn_game_a_top.Visible := true;
  btn_game_b_top.Visible := true;
  btn_time_top.Visible   := true;
  btn_acl_top.Visible    := true;
  btn_alarm_top.Visible  := true;

  btn_left_top.Picture := nil;  btn_left_top.Visible := true;
  btn_right_top.Picture := nil; btn_right_top.Visible := true;

  // controller overlay
  Vid_Display_Ctrl_Overlay;

  // sets mode to "ACL/2"
  iMode := 2; Gam_Set_Mode;
end;





procedure TForm1.im_background_openMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: PRESS                                                }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (Button = mbLeft) then
    begin
      iMouseX := X;
      iMouseY := Y;
    end;
end;





procedure TForm1.im_background_openMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: MOVE                                                 }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (ssLeft in Shift) then
    begin
      Top  := Top  + Y - iMouseY;
      Left := Left + X - iMouseX;
    end;
end;





procedure TForm1.btn_game_a_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Game A button: PRESS                                                         }
begin
  if Button = mbLeft then
    Inp_game_a_down;
end;





procedure TForm1.btn_game_a_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Game A button: RELEASE                                                       }
begin
  if Button = mbLeft then
    Inp_game_a_up;
end;





procedure TForm1.btn_game_b_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Game B button: PRESS                                                         }
begin
  if Button = mbLeft then
    Inp_game_b_down;
end;





procedure TForm1.btn_game_b_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Game B button: RELEASE                                                       }
begin
  if Button = mbLeft then
    Inp_game_b_up;
end;





procedure TForm1.btn_time_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Time button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_time_down;
end;





procedure TForm1.btn_time_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Time button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_time_up;
end;





procedure TForm1.btn_acl_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ ACL button: PRESS                                                            }
begin
  if Button = mbLeft then
    Inp_acl_down;
end;





procedure TForm1.btn_acl_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ ACL button: RELEASE                                                          }
begin
  if Button = mbLeft then
    Inp_acl_up;
end;





procedure TForm1.btn_ctrlr_overlay_nwMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ NW button: PRESS                                                             }
begin
  if Button = mbLeft then
    Inp_1_nw_down;
end;





procedure TForm1.btn_ctrlr_overlay_nwMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ NW button: RELEASE                                                           }
begin
  if Button = mbLeft then
    Inp_1_nw_up;
end;





procedure TForm1.btn_ctrlr_overlay_swMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ SW button: PRESS                                                             }
begin
  if Button = mbLeft then
    Inp_2_sw_down;
end;





procedure TForm1.btn_ctrlr_overlay_swMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ SW button: RELEASE                                                           }
begin
  if Button = mbLeft then
    Inp_2_sw_up;
end;





procedure TForm1.btn_ctrlr_overlay_neMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ NE button: PRESS                                                             }
begin
  if Button = mbLeft then
    Inp_3_ne_down;
end;





procedure TForm1.btn_ctrlr_overlay_neMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ NE button: RELEASE                                                           }
begin
  if Button = mbLeft then
    Inp_3_ne_up;
end;





procedure TForm1.btn_ctrlr_overlay_seMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ SE button: PRESS                                                             }
begin
  if Button = mbLeft then
    Inp_4_se_down;
end;





procedure TForm1.btn_ctrlr_overlay_seMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ SE button: RELEASE                                                           }
begin
  if Button = mbLeft then
    Inp_4_se_up;
end;





procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: PRESS                                                              }
begin
  case Key of
    VK_UP:    bUpPress    := true;                                              // arrow up
    VK_DOWN:  bDownPress  := true;                                              // arrow down
    VK_LEFT:  bLeftPress  := true;                                              // arrow left
    VK_RIGHT: bRightPress := true;                                              // arrow right
  end; // case..of

  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[1].word then Inp_game_a_down else                                 // Game A
  if Key = aKeys[2].word then Inp_game_b_down else                                 // Game B
  if Key = aKeys[3].word then Inp_time_down   else                                 // Time
  if Key = aKeys[4].word then Inp_acl_down    else                                 // ACL
  if (Key = aKeys[5].word) or (bLeftPress  and bUpPress)   then Inp_1_nw_down else // Luigi up
  if (Key = aKeys[6].word) or (bLeftPress  and bDownPress) then Inp_2_sw_down else // Luigi down
  if (Key = aKeys[7].word) or (bRightPress and bUpPress)   then Inp_3_ne_down else // Mario up
  if (Key = aKeys[8].word) or (bRightPress and bDownPress) then Inp_4_se_down;     // Mario down
end;





procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: RELEASE                                                            }
begin
  case Key of
    VK_UP:    begin bUpPress    := false; Inp_1_nw_up; Inp_3_ne_up; end;        // arrow up
    VK_DOWN:  begin bDownPress  := false; Inp_2_sw_up; Inp_4_se_up; end;        // arrow down
    VK_LEFT:  begin bLeftPress  := false; Inp_1_nw_up; Inp_2_sw_up; end;        // arrow left
    VK_RIGHT: begin bRightPress := false; Inp_3_ne_up; Inp_4_se_up; end;        // arrow right
  end; // case..of

  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[1].word then Inp_game_a_up else                                // Game A
  if Key = aKeys[2].word then Inp_game_b_up else                                // Game B
  if Key = aKeys[3].word then Inp_time_up   else                                // Time
  if Key = aKeys[4].word then Inp_acl_up    else                                // ACL
  if Key = aKeys[5].word then Inp_1_nw_up   else                                // Luigi up
  if Key = aKeys[6].word then Inp_2_sw_up   else                                // Luigi down
  if Key = aKeys[7].word then Inp_3_ne_up   else                                // Mario up
  if Key = aKeys[8].word then Inp_4_se_up;                                      // Mario down
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Inp_game_a_down;
{ Game A button: PRESS                                                         }
begin
  btn_game_a_down.Visible := true and im_background_open.Visible;

  if (iMode = 2) or                                                             // ACL/2     or
     (iMode = 4) or                                                             // Time      or
     (iMode = 9) then                                                           // Game over ->
    begin
      iMode := 5; Gam_Set_Mode;                                                   //         -> HiScore A
    end;
end;





procedure TForm1.Inp_game_a_up;
{ Game A button: RELEASE                                                       }
begin
  btn_game_a_down.Visible := false;

  if iMode = 5 then                                                             // HiScore A ->
    begin
      iMode := 6; Gam_Set_Mode;                                                   //         -> Game A
    end;
end;





procedure TForm1.Inp_game_b_down;
{ Game B button: PRESS                                                         }
begin
  btn_game_b_down.Visible := true and im_background_open.Visible;

  if (iMode = 2) or                                                             // ACL/2     or
     (iMode = 4) or                                                             // Time      or
     (iMode = 9) then                                                           // Game over ->
    begin
      iMode := 7; Gam_Set_Mode;                                                   //         -> HiScore B
    end;
end;





procedure TForm1.Inp_game_b_up;
{ Game B button: RELEASE                                                       }
begin
  btn_game_b_down.Visible := false;

  if iMode = 7 then                                                             // HiScore B ->
    begin
      iMode := 8; Gam_Set_Mode;                                                   //         -> Game B
    end;
end;





procedure TForm1.Inp_time_down;
{ Time button: PRESS                                                           }
begin
  btn_time_down.Visible := true and im_background_open.Visible;

  if (iMode = 2) or                                                             // ACL/2     or
     (iMode = 4) or                                                             // Time      or
     (iMode = 9) then                                                           // Game over ->
    begin
      iMode := 3; Gam_Set_Mode;                                                   //         -> Alarm
    end;
end;





procedure TForm1.Inp_time_up;
{ Time button: RELEASE                                                         }
begin
  btn_time_down.Visible := false;

  if iMode = 3 then                                                             // Alarm ->
    begin
      iMode := 4; Gam_Set_Mode;                                                   //     -> Time
    end;
end;





procedure TForm1.Inp_acl_down;
{ ACL button: PRESS                                                            }
begin
  btn_acl_down.Visible := true and im_background_open.Visible;

  if iMode = 0 then                                                             // closed handheld game ->
    im_background_closedClick(Self)                                               //                    -> open handheld game
  else                                                                          // open handheld game ->
    begin
      iMode := 1; Gam_Set_Mode;                                                   //                  -> ACL/1
    end;
end;





procedure TForm1.Inp_acl_up;
{ ACL button: RELEASE                                                          }
begin
  btn_acl_down.Visible := false;

  if iMode = 1 then                                                             // ACL/1 ->
    begin
      iMode := 2; Gam_Set_Mode;                                                   //     -> ACL/2
    end;
end;





procedure TForm1.Inp_1_nw_down;
{ NW button (Luigi up): PRESS                                                  }
begin
  if im_background_open.Visible then
    btn_left_top.Picture := btn_left_bottom_up.Picture;

  if bControllerLuigi and (not bNWKeyPressed) then
    begin
      bNWKeyPressed := true;

      if iPosLuigi < 3 then
        begin
          Gam_Move_Luigi_Up;

          Timer_Move_Luigi_Up.Interval := 250;
          Timer_Move_Luigi_Up.Enabled := true;
        end;
    end;
end;





procedure TForm1.Inp_1_nw_up;
{ NW button (Luigi up): RELEASE                                                }
begin
  btn_left_top.Picture := nil;
  bNWKeyPressed := false;

  Timer_Move_Luigi_Up.Enabled := false;
end;





procedure TForm1.Inp_2_sw_down;
{ SW button (Luigi down): PRESS                                                }
begin
  if im_background_open.Visible then
    btn_left_top.Picture := btn_left_bottom_down.Picture;

  if bControllerLuigi and (not bSWKeyPressed) then
    begin
      bSWKeyPressed := true;

      if iPosLuigi > 1 then
        begin
          Gam_Move_Luigi_Down;

          Timer_Move_Luigi_Down.Interval := 250;
          Timer_Move_Luigi_Down.Enabled := true;
        end;
    end;
end;





procedure TForm1.Inp_2_sw_up;
{ SW button (Luigi down): RELEASE                                              }
begin
  btn_left_top.Picture := nil;
  bSWKeyPressed := false;

  Timer_Move_Luigi_Down.Enabled := false;
end;





procedure TForm1.Inp_3_ne_down;
{ NE button (Mario up): PRESS                                                  }
begin
  if im_background_open.Visible then
    btn_right_top.Picture := btn_right_bottom_up.Picture;

  if bControllerMario and (not bNEKeyPressed) then
    begin
      bNEKeyPressed := true;

      if iPosMario < 3 then
        begin
          Gam_Move_Mario_Up;

          Timer_Move_Mario_Up.Interval := 250;
          Timer_Move_Mario_Up.Enabled := true;
        end;
    end;
end;





procedure TForm1.Inp_3_ne_up;
{ NE button (Mario up): RELEASE                                                }
begin
  btn_right_top.Picture := nil;
  bNEKeyPressed := false;

  Timer_Move_Mario_Up.Enabled := false;
end;





procedure TForm1.Inp_4_se_down;
{ SE button (Mario down): PRESS                                                }
begin
  if im_background_open.Visible then
    btn_right_top.Picture := btn_right_bottom_down.Picture;

  if bControllerMario and (not bSEKeyPressed) then
    begin
      bSEKeyPressed := true;

      if iPosMario > 1 then
        begin
          Gam_Move_Mario_Down;

          Timer_Move_Mario_Down.Interval := 250;
          Timer_Move_Mario_Down.Enabled := true;
        end;
    end;
end;





procedure TForm1.Inp_4_se_up;
{ SE button (Mario down): RELEASE                                              }
begin
  btn_right_top.Picture := nil;
  bSEKeyPressed := false;

  Timer_Move_Mario_Down.Enabled := false;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME ROUTINES                                                * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Gam_Stop_All_Timers;
{ stops action                                                                 }
begin
  Timer_Bonus.Enabled            := false;
  Timer_Box_To_Van.Enabled       := false;
  Timer_Chance.Enabled           := false;
  Timer_Clock.Enabled            := false;
  Timer_Game.Enabled             := false;
  Timer_Game_Over.Enabled        := false;
  Timer_Luigi_Moved_Box.Enabled  := false;
  Timer_Luigi_Moving_Box.Enabled := false;
  Timer_Mario_Moved_Box.Enabled  := false;
  Timer_Mario_Moving_Box.Enabled := false;
  Timer_Miss.Enabled             := false;
  Timer_Move_Luigi_Down.Enabled  := false;
  Timer_Move_Luigi_Up.Enabled    := false;
  Timer_Move_Mario_Down.Enabled  := false;
  Timer_Move_Mario_Up.Enabled    := false;
  Timer_Score_Blink.Enabled      := false;
  Timer_Van_Smoke.Enabled        := false;
end;





procedure TForm1.Gam_Check_Collision_Luigi;
{ checks whether a Luigi has got a box                                         }
begin
  if (iPosLuigi = 1) and aIm_box[0, 1].Visible then
    begin
      bControllerLuigi := false;

      iLoopMovingBoxCode := 2;
      Timer_Luigi_Moving_Box.Interval := 150;
      Timer_Luigi_Moving_Box.Enabled := true;

      Timer_Game.Interval := Timer_Game.Interval + Timer_Luigi_Moving_Box.Interval;
    end;

  if (iPosLuigi = 2) and aIm_box[0, 3].Visible then
    begin
      bControllerLuigi := false;

      iLoopMovingBoxCode := 4;
      Timer_Luigi_Moving_Box.Interval := 150;
      Timer_Luigi_Moving_Box.Enabled := true;

      Timer_Game.Interval := Timer_Game.Interval + Timer_Luigi_Moving_Box.Interval;
    end;

  if (iPosLuigi = 3) and aIm_box[0, 5].Visible then
    begin
      bControllerLuigi := false;

      iLoopMovingBoxCode := 6;
      Timer_Luigi_Moving_Box.Interval := 150;
      Timer_Luigi_Moving_Box.Enabled := true;

      Timer_Game.Interval := Timer_Game.Interval + Timer_Luigi_Moving_Box.Interval;
    end;
end;





procedure TForm1.Gam_Check_Collision_Mario;
{ checks whether a Mario has got a box                                         }
begin
  if (iPosMario = 1) and aIm_box[10, 1].Visible then
    begin
      bControllerMario := false;

      iLoopMovingBoxCode := 1;
      Timer_Mario_Moving_Box.interval := 150;
      Timer_Mario_Moving_Box.Enabled := true;

      Timer_Game.Interval := Timer_Game.Interval + Timer_Mario_Moving_Box.Interval;
    end;

  if (iPosMario = 2) and aIm_box[9, 2].Visible then
    begin
      bControllerMario := false;

      iLoopMovingBoxCode := 3;
      Timer_Mario_Moving_Box.interval := 150;
      Timer_Mario_Moving_Box.Enabled := true;

      Timer_Game.Interval := Timer_Game.Interval + Timer_Mario_Moving_Box.Interval;
    end;

  if (iPosMario = 3) and aIm_box[9, 4].Visible then
    begin
      bControllerMario := false;

      iLoopMovingBoxCode := 5;
      Timer_Mario_Moving_Box.interval := 150;
      Timer_Mario_Moving_Box.Enabled := true;

      Timer_Game.Interval := Timer_Game.Interval + Timer_Mario_Moving_Box.Interval;
    end;
end;





procedure TForm1.Gam_Move_Luigi_Down;
{ Luigi moves down one step                                                    }
begin
  Timer_Luigi_Moved_Box.Enabled := false;
  Timer_Luigi_Moving_Box.Enabled := false;

  Vid_Show_Luigi(iPosLuigi, false);
  Dec(iPosLuigi);
  Vid_Show_Luigi(iPosLuigi, true);

  Snd_Play(pfs_movechar);

  Gam_Check_Collision_Luigi;
end;





procedure TForm1.Gam_Move_Luigi_Up;
{ Luigi moves up one step                                                      }
begin
  Timer_Luigi_Moved_Box.Enabled := false;
  Timer_Luigi_Moving_Box.Enabled := false;

  Vid_Show_Luigi(iPosLuigi, false);
  Inc(iPosLuigi);
  Vid_Show_Luigi(iPosLuigi, true);

  Snd_Play(pfs_movechar);

  Gam_Check_Collision_Luigi;
end;





procedure TForm1.Gam_Move_Mario_Down;
{ Mario moves down one step                                                    }
begin
  Timer_Mario_Moved_Box.Enabled := false;
  Timer_Mario_Moving_Box.Enabled := false;

  Vid_Show_Mario(iPosMario, false);
  Dec(iPosMario);
  Vid_Show_Mario(iPosMario, true);

  Snd_Play(pfs_movechar);

  Gam_Check_Collision_Mario;
end;





procedure TForm1.Gam_Move_Mario_Up;
{ Mario moves up one step                                                      }
begin
  Timer_Mario_Moved_Box.Enabled := false;
  Timer_Mario_Moving_Box.Enabled := false;

  Vid_Show_Mario(iPosMario, false);
  Inc(iPosMario);
  Vid_Show_Mario(iPosMario, true);

  Snd_Play(pfs_movechar);

  Gam_Check_Collision_Mario;
end;





procedure TForm1.Gam_Set_Max_Boxes_On_Screen;
{ calculates difficulty increase, depending on current score                   }
  var
    iHundreds, iTenths: integer;
begin
  iHundreds := iScore div 100;
  iTenths := (iScore mod 100) div 20; // changes at every 20 points

  iMaxBoxesOnScreen := aMaxBoxes[iMode, iHundreds, iTenths];
end;





procedure TForm1.Gam_Set_Mode;
{ sets game/time/ACL mode, depending on the value of the iMode variable        }
begin
  case iMode of
    0: begin                                                                    // handheld is closed
       end;
    1: begin                                                                    // ACL/1 (ACL button press)
         Gam_Stop_All_Timers;
         Snd_Silence;

         bControllerLuigi := false;
         bControllerMario := false;
       end;
    2: Vid_Show_All(true);                                                      // ACL/2 (ACL button release)
    3: begin                                                                    // ALARM (Time button press)
         Gam_Stop_All_Timers;

         Vid_Show_All(false);
         aIm_van[1].Visible := true;
         im_panel_time_colon.Visible := true;
         im_panel_time_pm.Visible := true;
         aIm_panel_digit_pos[1].Picture := aIm_panel_number[1].Picture;
         aIm_panel_digit_pos[2].Picture := aIm_panel_number[2].Picture;
         aIm_panel_digit_pos[3].Picture := aIm_panel_number[0].Picture;
         aIm_panel_digit_pos[4].Picture := aIm_panel_number[0].Picture;

         bControllerLuigi := false;
         bControllerMario := false;
       end;
    4: begin                                                                    // TIME (Time button release)
         Gam_Stop_All_Timers;

         Vid_Show_All(false);
         aIm_van[1].Visible := true;
         im_panel_time_colon.Visible := true;

         Timer_Clock.Interval := 1;
         Timer_Clock.Enabled := true;

         bControllerLuigi := false;
         bControllerMario := false;
       end;
 5, 7: begin                                                                    // HISCORE A/B (Game A/B button press)
         Gam_Stop_All_Timers;

         Vid_Show_All(false);
         aIm_van[1].Visible := true;

         case iMode of
           5: begin im_panel_game_a.Visible := true; Vid_Show_Panel_Score(iHiScoreA, true); end;
           7: begin im_panel_game_b.Visible := true; Vid_Show_Panel_Score(iHiScoreB, true); end;
         end; // case..of

         iPosLuigi := 1; Vid_Show_Luigi(iPosLuigi, true);
         iPosMario := 2; Vid_Show_Mario(iPosMario, true);

         bControllerLuigi := false;
         bControllerMario := false;
       end;
 6, 8: begin                                                                    // GAME A/B (Game A/B button release)
         iMiss := 0;
         iBoxesOnVan := 0;
         iScoreMultiplier := 1;
         iScore := 0; Vid_Show_Panel_Score(iScore, true);
         bControllerLuigi := true;
         bControllerMario := true;
         iLoopBonus := 0;
         iLoopBoxToVan := 0;

         iLoopGame := 1;
         Gam_Set_Timer_Game_Interval;
         Timer_Game.Enabled := true;
       end;
    9: begin                                                                    // GAME OVER
       end;
  end; // case..of
end;





procedure TForm1.Gam_Set_Timer_Game_Interval;
{ calculates difficulty increase, depending on current score                   }
  var
    iTempScore,
    iTempHundreds,
    iTempSpeed: integer;
begin
  if iScore >= 1000 then
    iTempSpeed := 140
  else
    begin
      iTempScore    := iScore;
      iTempHundreds := 0;

      while iTempScore > 99 do
        begin
          Dec(iTempScore, 100);
          Inc(iTempHundreds);
        end;

      case iMode of
        6: begin                                                                // game A
             if  iTempScore <  20                         then iTempSpeed := 430;     (*OK*)
             if (iTempScore >= 20) and (iTempScore <= 39) then iTempSpeed := 415;     (*OK*)
             if (iTempScore >= 40) and (iTempScore <= 59) then iTempSpeed := 400;     (*OK*)
             if (iTempScore >= 60) and (iTempScore <= 79) then iTempSpeed := 375;     (*OK*)
             if (iTempScore >= 80) and (iTempScore <= 99) then iTempSpeed := 350;     (*OK*)

             Dec(iTempSpeed, iTempHundreds * 25);
           end;
        8: begin                                                                // game B
             if  iTempScore <  20                         then iTempSpeed := 400;     (*OK*)
             if (iTempScore >= 20) and (iTempScore <= 39) then iTempSpeed := 375;     (*OK*)
             if (iTempScore >= 40) and (iTempScore <= 59) then iTempSpeed := 350;     (*OK*)
             if (iTempScore >= 60) and (iTempScore <= 79) then iTempSpeed := 335;     (*OK*)
             if (iTempScore >= 80) and (iTempScore <= 99) then iTempSpeed := 320;     (*OK*)

             Dec(iTempSpeed, iTempHundreds * 20);
           end;
      end; // case..of
    end;

  Timer_Game.Interval := iTempSpeed;
end;





procedure TForm1.Gam_Start_Chance_Time;
{ "chance time" initialization                                                 }
begin
  Gam_Stop_All_Timers;
  Snd_Silence;

  bControllerLuigi := false;
  bControllerMario := false;

  iLoopChance := 1;
  Timer_Chance.Interval := 300;
  Timer_Chance.Enabled := true;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: TIMERS                                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Timer_BonusTimer(Sender: TObject);
{ "bonus" intermission                                                         }
  var
    i: integer;
begin
  case iLoopBonus of
    1: begin                                                                    // driver moves inside the van
         aIm_van[2].Visible := false;

         Timer_Bonus.Interval := 150;
       end;
   { 2 to 11: score increases }
   12: begin                                                                    // van disappears, leaving smoke behind it
         Timer_Van_Smoke.Enabled := false;

         Vid_Show_All_Boxes_On_Van(false);

         aIm_van[1].Visible := false;
         aIm_van[3].Visible := true;
         aIm_van[5].Visible := true;
         aIm_van[6].Visible := true;

         Timer_Bonus.Interval := 650;
       end;
   13: begin                                                                    // smoke animation
         aIm_van[4].Visible := true;
         aIm_van[5].Visible := false;
       end;
   14: begin                                                                    // smoke animation
         aIm_van[4].Visible := false;
       end;
   15: begin                                                                    // smoke animation
         aIm_van[3].Visible := false;
         aIm_van[5].Visible := true;
         aIm_van[6].Visible := false;
       end;
   16:   aIm_van[5].Visible := false;                                           // smoke disappears
   17: begin                                                                    // Luigi and Mario take a rest
         // displays Luigi and Mario taking a rest
         Vid_Show_All_Luigi(false);
         Vid_Show_All_Mario(false);

         aIm_bonus[1, 1].Visible := true; aIm_bonus[2, 1].Visible := true;

         // removes the boxes close to the edge of conveyor belts
         aIm_box[ 0, 1].Visible := false; aIm_box[ 1, 1].Visible := false;
         aIm_box[ 0, 3].Visible := false; aIm_box[ 1, 3].Visible := false;
         aIm_box[ 0, 5].Visible := false; aIm_box[ 1, 5].Visible := false;
         aIm_box[ 8, 2].Visible := false; aIm_box[ 9, 2].Visible := false;
         aIm_box[ 8, 4].Visible := false; aIm_box[ 9, 4].Visible := false;
         aIm_box[10, 1].Visible := false; aIm_box[11, 1].Visible := false;

         Timer_Bonus.Interval := 800;
       end;
   18: begin                                                                    // bonus animation
         aIm_bonus[1, 2].Visible := true;  aIm_bonus[2, 2].Visible := true;
       end;
   19: begin                                                                    // bonus animation
         aIm_bonus[1, 2].Visible := false; aIm_bonus[2, 2].Visible := false;
       end;
   20: begin                                                                    // bonus animation
         aIm_bonus[1, 2].Visible := true;  aIm_bonus[2, 2].Visible := true;
       end;
   21: begin                                                                    // bonus animation
         aIm_bonus[1, 2].Visible := false; aIm_bonus[2, 2].Visible := false;
       end;
   22: aIm_van[1].Visible := true;                                              // van has came back empty
   23: begin                                                                    // foremen appear
         aIm_bonus[1, 1].Visible := false; aIm_bonus[2, 1].Visible := false;

         aIm_miss[1, 2].Visible := true; aIm_miss[1, 3].Visible := true;
         aIm_miss[2, 3].Visible := true; aIm_miss[2, 4].Visible := true;
       end;
   24: begin                                                                    // game reprises
         iBoxesOnVan := 0; // van is empty

         // displays main characters "in game"
         Vid_Show_All_Miss(false);
         Vid_Show_Luigi(iPosLuigi, true);
         Vid_Show_Mario(iPosMario, true);

         bControllerLuigi := true;
         bControllerMario := true;

         // game reprises
         Gam_Set_Timer_Game_Interval;
         Timer_Game.Enabled := true;

         iLoopBonus := -1; // becomes 0
         Timer_Bonus.Enabled := false;
       end;
  end; // case..of

  // increases score by 10 points (20 when in "chance time")
  if (iLoopBonus > 1) and (iLoopBonus < 12) then
    begin
      Inc(iScore, iScoreMultiplier); Vid_Show_Panel_Score(iScore, true);

      // enters in "chance time" mode or plays sound sample as score increases
      if (iScore = 300) then
        Gam_Start_Chance_Time
      else
        Snd_Play(pfs_movebox);
    end;

  Inc(iLoopBonus);
end;





procedure TForm1.Timer_Box_To_VanTimer(Sender: TObject);
begin
  if Odd(iBoxesOnVan) then                                                      // there's already 1, 3, 5 or 7 boxes on the van
    case iLoopBoxToVan
      of 1: begin aIm_van_box[2, 1].Visible := false; aIm_van_box[1, 1].Visible := true; Vid_Show_Luigi(iPosLuigi, true); end;
         2: begin aIm_van_box[1, 1].Visible := false; aIm_van_box[1, 2].Visible := true; if iBoxesOnVan = 7 then begin Timer_Box_To_Van.Enabled := false; iLoopBoxToVan := -1; { becomes 0 } end; end;
         3: begin aIm_van_box[1, 2].Visible := false; aIm_van_box[1, 3].Visible := true; if iBoxesOnVan = 5 then begin Timer_Box_To_Van.Enabled := false; iLoopBoxToVan := -1; { becomes 0 } end; end;
         4: begin aIm_van_box[1, 3].Visible := false; aIm_van_box[1, 4].Visible := true; if iBoxesOnVan = 3 then begin Timer_Box_To_Van.Enabled := false; iLoopBoxToVan := -1; { becomes 0 } end; end;
         5: begin aIm_van_box[1, 4].Visible := false; aIm_van_box[1, 5].Visible := true;                               Timer_Box_To_Van.Enabled := false; iLoopBoxToVan := -1; { becomes 0 } end;
    end // case..of
  else                                                                          // there's already 1, 3, 5 or 7 boxes on the van
    case iLoopBoxToVan
      of 1: begin
              aIm_van_box[2, 1].Visible := false; aIm_van_box[2, 2].Visible := true; Vid_Show_Luigi(iPosLuigi, true);

              if iBoxesOnVan = 6 then
                begin
                  // driver appears and starts the engine
                  aIm_van[2].Visible := true;

                  Timer_Van_Smoke.Interval := 150;
                  Timer_Van_Smoke.Enabled := true;
                end;

              if iBoxesOnVan = 8 then
                begin
                  // "bonus" intermission starts
                  bControllerLuigi := false;
                  bControllerMario := false;

                  Timer_Luigi_Moved_Box.Enabled := false;
                  Timer_Mario_Moved_Box.Enabled := false;
                  Timer_Game.Enabled := false;
                  Timer_Box_To_Van.Enabled := false;

                  iLoopBoxToVan := -1; // becomes 0

                  iLoopBonus := 1;
                  Timer_Bonus.Interval := 350;
                  Timer_Bonus.Enabled := true;
                end;
            end;
         2: begin aIm_van_box[2, 2].Visible := false; aIm_van_box[2, 3].Visible := true; if iBoxesOnVan = 6 then begin Timer_Box_To_Van.Enabled := false; iLoopBoxToVan := -1; { becomes 0 } end; end;
         3: begin aIm_van_box[2, 3].Visible := false; aIm_van_box[2, 4].Visible := true; if iBoxesOnVan = 4 then begin Timer_Box_To_Van.Enabled := false; iLoopBoxToVan := -1; { becomes 0 } end; end;
         4: begin aIm_van_box[2, 4].Visible := false; aIm_van_box[2, 5].Visible := true; if iBoxesOnVan = 2 then begin Timer_Box_To_Van.Enabled := false; iLoopBoxToVan := -1; { becomes 0 } end; end;
    end; // case..of

  Inc(iLoopBoxToVan);
end;





procedure TForm1.Timer_ChanceTimer(Sender: TObject);
{ "chance time" starts here!                                                   }
  var
    i: integer;
begin
  if iLoopChance < 10 then
    begin                                                                       // intermission
      if Odd(iLoopChance) then
        begin
          i := 0; Vid_Show_Panel_Miss(i);
        end
      else
        if iMiss > 0 then
          Vid_Show_Panel_Miss(iMiss);

      Snd_Play(pfs_miss1);
    end
  else
    begin                                                                       // game reprises
      // chance time is set
      if iMiss = 0 then
        begin
          iScoreMultiplier := 2;

          iLoopScoreBlink := 1;
          Timer_Score_Blink.Interval := 500;
          Timer_Score_Blink.Enabled := true;
        end;

      // resets misses
      iMiss := 0;

      // box falls into van if previously falling
      if iLoopBoxToVan <> 0 then
        begin
          Timer_Box_To_Van.Interval := 200;
          Timer_Box_To_Van.Enabled := true;
        end;

      // back to bonus intermission if previously active
      if iLoopBonus <> 0 then
        Timer_Bonus.Enabled := true
      else                                                                      
        // game reprises
        begin
          Timer_Luigi_Moved_Box.Interval := 200;
          Timer_Luigi_Moved_Box.Enabled := true;

          Timer_Mario_Moved_Box.Interval := 200;
          Timer_Mario_Moved_Box.Enabled := true;

          Gam_Set_Timer_Game_Interval;
          Timer_Game.Enabled := true;
        end;

      Timer_Chance.Enabled := false;
    end;

  Inc(iLoopChance);
end;





procedure TForm1.Timer_ClockTimer(Sender: TObject);
{ transforms current time into "sprites" form, then displays it                }
  var
    present:              TDateTime;
    hour, min, sec, msec: Word;
begin
  present := Now;                                                               // gets system time...
  DecodeTime(present, hour, min, sec, msec);                                    // and decodes it

  if hour > 11 then                                                             // changes mode from 0/24 to 12 AM/12 PM
    begin
      Dec(hour, 12);
      im_panel_time_am.Visible := false;
      im_panel_time_pm.Visible := true;
    end
  else
    begin
      im_panel_time_am.Visible := true;
      im_panel_time_pm.Visible := false;
    end;

  if hour = 0 then                                                              // changes mode from 0/24 to 12 AM/12 PM
    hour := 12;

  if hour > 9 then                                                              // hour: tens
    aIm_panel_digit_pos[1].Picture := aIm_panel_number[hour div 10].Picture
  else
    aIm_panel_digit_pos[1].Picture := nil;

  aIm_panel_digit_pos[2].Picture := aIm_panel_number[hour mod 10].Picture;      // hour: units
  aIm_panel_digit_pos[3].Picture := aIm_panel_number[min  div 10].Picture;      // minute: tens
  aIm_panel_digit_pos[4].Picture := aIm_panel_number[min  mod 10].Picture;      // minute: units

  Timer_Clock.Interval := 1000;
end;





procedure TForm1.Timer_Game_OverTimer(Sender: TObject);
{ mode goes back into "time", five minutes after game is over                  }
begin
  if iLoopGameOver <= 300 then
    begin
      aIm_miss[2, 4].Visible :=     Odd(iLoopMissCode) and not Odd(iLoopGameOver);
      aIm_miss[2, 5].Visible :=     Odd(iLoopMissCode) and     Odd(iLoopGameOver);
      aIm_miss[1, 3].Visible := not Odd(iLoopMissCode) and not Odd(iLoopGameOver);
      aIm_miss[1, 4].Visible := not Odd(iLoopMissCode) and     Odd(iLoopGameOver);

      Inc(iLoopGameOver);
    end
  else
    begin
      iMode := 4; Gam_Set_Mode;

      Timer_Game_Over.Enabled := false;
    end;
end;





procedure TForm1.Timer_GameTimer(Sender: TObject);
  var
    i, j,
    iOnScreen  : integer;

    bGenerate,
    bDoneMiss,
    bPlaySound : boolean;
begin
  if Odd(iLoopGame) then
    // Luigi's side
    begin
      // checks collision for misses
      bDoneMiss := false;

      if                      aIm_box[0, 1].Visible and (iPosLuigi <> 1)  then begin iLoopMissCode := 2; bDoneMiss := true; end;
      if (not bDoneMiss) and (aIm_box[0, 3].Visible and (iPosLuigi <> 2)) then begin iLoopMissCode := 4; bDoneMiss := true; end;
      if (not bDoneMiss) and (aIm_box[0, 5].Visible and (iPosLuigi <> 3)) then begin iLoopMissCode := 6; bDoneMiss := true; end;

      // boxes moving on conveyor belts (3, 5, right side of 1)
      if not bDoneMiss then
        begin
          for i := 0 to 8 do
            begin
              aIm_box[i, 1].Visible := aIm_box[i + 1, 1].Visible;
              aIm_box[i, 3].Visible := aIm_box[i + 1, 3].Visible;
              aIm_box[i ,5].Visible := aIm_box[i + 1, 5].Visible;
            end;

          aIm_box[9, 1].Visible := false;
          aIm_box[9, 3].Visible := false;
          aIm_box[9, 5].Visible := false;

          // checks collision Luigi<->boxes
          Gam_Check_Collision_Luigi;

          // plays audio sample?
          bPlaySound := false;
          for i := 0 to 7 do if aIm_box[i, 5].Visible then bPlaySound := true;
          for i := 0 to 7 do if aIm_box[i, 3].Visible then bPlaySound := true;
          for i := 0 to 7 do if aIm_box[i, 1].Visible then bPlaySound := true;

          if bPlaySound then
            Snd_Play(pfs_movebox);
        end;
    end
  else
    // Mario's side
    begin
      // checks collision for misses
      bDoneMiss := false;

      if                      aIm_box[10, 1].Visible and (iPosMario <> 1)  then begin iLoopMissCode := 1; bDoneMiss := true; end;
      if (not bDoneMiss) and (aIm_box[ 9, 2].Visible and (iPosMario <> 2)) then begin iLoopMissCode := 3; bDoneMiss := true; end;
      if (not bDoneMiss) and (aIm_box[ 9, 4].Visible and (iPosMario <> 3)) then begin iLoopMissCode := 5; bDoneMiss := true; end;

      // boxes moving on conveyor belts (2, 4, left side of 1)
      if not bDoneMiss then
        begin
          for i := 10 to 12   do aIm_box[i, 1].Visible := aIm_box[i + 1, 1].Visible;
          for i := 9 downto 1 do aIm_box[i, 2].Visible := aIm_box[i - 1, 2].Visible;
          for i := 9 downto 1 do aIm_box[i, 4].Visible := aIm_box[i - 1, 4].Visible;

          aIm_box[0, 2].Visible := false;
          aIm_box[0, 4].Visible := false;

          // generates new box
          bGenerate := false;

          if random(2) = 0 then
            bGenerate := true;

          if   aIm_box[10, 1].Visible
            or aIm_box[11, 1].Visible
            or aIm_box[ 7, 2].Visible
            or aIm_box[ 7, 4].Visible
            then bGenerate := false;

          iOnScreen := 0;

          for i := 0 to 9 do
            for j := 1 to 5 do
              if aIm_box[i, j].Visible
                then Inc(iOnScreen);

          for i := 10 to 12 do
            if aIm_box[i, 1].Visible then
              Inc(iOnScreen);

          Gam_Set_Max_Boxes_On_Screen;

          if iOnScreen > iMaxBoxesOnScreen then
            bGenerate := false;

          if iOnScreen = 0 then
            bGenerate := true;

          // actual generation of new box
          aIm_box[12, 1].Visible := bGenerate;

          // checks collision Mario<->boxes
          Gam_Check_Collision_Mario;

          // plays audio sample?
          bPlaySound := false;

          for i :=  2 to  9 do if aIm_box[i, 4].Visible then bPlaySound := true;
          for i :=  2 to  9 do if aIm_box[i, 2].Visible then bPlaySound := true;
          for i := 10 to 12 do if aIm_box[i, 1].Visible then bPlaySound := true;

          if bPlaySound then
            Snd_Play(pfs_movebox);
        end;
    end;

  // miss occured, or proceed to next tick?
  if bDoneMiss then
    // a miss has occured
    begin
      Gam_Stop_All_Timers;

      for i := 1 to 4 do
        aIm_panel_digit_pos[i].Visible := true;

      bControllerLuigi := false;
      bControllerMario := false;

      iLoopMiss := 1;
      Timer_Miss.Interval := 1;
      Timer_Miss.Enabled := true;
    end
  else
    // game continues...
    begin
      bControllerLuigi := true;
      bControllerMario := true;

      Gam_Set_Timer_Game_Interval;
    end;

  // loops action
  Inc(iLoopGame);

  if iLoopGame = 3 then
    iLoopGame := 1;
end;





procedure TForm1.Timer_Luigi_Moved_BoxTimer(Sender: TObject);
{ Luigi moved a box -> hands down and enables controller again                 }
begin
  Vid_Show_Luigi(iPosLuigi, true);
  bControllerLuigi := true;

  Timer_Luigi_Moved_Box.Enabled := false;
end;





procedure TForm1.Timer_Luigi_Moving_BoxTimer(Sender: TObject);
{ Luigi is moving box to upper conveyor belt                                   }
begin
  case iLoopMovingBoxCode of
    2: begin
         aIm_box[0, 1].Visible := false; aIm_hero_l[1, 2].Visible := false;
         aIm_box[0, 2].Visible := true;  aIm_hero_l[1, 3].Visible := true;

         Inc(iScore, iScoreMultiplier); Vid_Show_Panel_Score(iScore, true);

         Timer_Luigi_Moved_Box.Interval := 200;
         Timer_Luigi_Moved_Box.Enabled := true;
       end;
    4: begin
         aIm_box[0, 3].Visible := false; aIm_hero_l[2, 2].Visible := false;
         aIm_box[0, 4].Visible := true;  aIm_hero_l[2, 3].Visible := true;

         Inc(iScore, iScoreMultiplier); Vid_Show_Panel_Score(iScore, true);

         Timer_Luigi_Moved_Box.Interval := 200;
         Timer_Luigi_Moved_Box.Enabled := true;
       end;
    6: begin
         aIm_box[0, 5].Visible := false; aIm_hero_l[3, 2].Visible := false;
                                         aIm_hero_l[3, 3].Visible := true;

         aIm_van_box[2, 1].Visible := true;

         Inc(iScore, iScoreMultiplier); Vid_Show_Panel_Score(iScore, true);

         Timer_Luigi_Moved_Box.Interval := 200;
         Timer_Luigi_Moved_Box.Enabled := true;

         Inc(iBoxesOnVan);

         iLoopBoxToVan := 1;
         Timer_Box_To_Van.Interval := 200;
         Timer_Box_To_Van.Enabled := true;
       end;
  end; // case..of

  // "chance time" starts, or game proceeds with beep
  if (iScore = 300) then
    Gam_Start_Chance_Time
  else
    Snd_Play(pfs_point);

  Timer_Luigi_Moving_Box.Enabled := false;
end;





procedure TForm1.Timer_Mario_Moved_BoxTimer(Sender: TObject);
{ Mario moved a box -> hands down and enables controller again                 }
begin
  Vid_Show_Mario(iPosMario, true);
  bControllerMario := true;

  Timer_Mario_Moved_Box.Enabled := false;
end;





procedure TForm1.Timer_Mario_Moving_BoxTimer(Sender: TObject);
{ Mario is moving box to upper conveyor belt                                   }
begin
  case iLoopMovingBoxCode of
    1: begin
         aIm_box[10, 1].Visible := false; aIm_hero_r[1, 2].Visible := false;
         aIm_box[ 9, 1].Visible := true;  aIm_hero_r[1, 3].Visible := true;

         Inc(iScore, iScoreMultiplier); Vid_Show_Panel_Score(iScore, true);

         Timer_Mario_Moved_Box.Interval := 200;
         Timer_Mario_Moved_Box.Enabled := true;
       end;
    3: begin
         aIm_box[ 9, 2].Visible := false; aIm_hero_r[2, 2].Visible := false;
         aIm_box[ 9, 3].Visible := true;  aIm_hero_r[2, 3].Visible := true;

         Inc(iScore, iScoreMultiplier); Vid_Show_Panel_Score(iScore, true);

         Timer_Mario_Moved_Box.Interval := 200;
         Timer_Mario_Moved_Box.Enabled := true;
       end;
    5: begin
         aIm_box[ 9, 4].Visible := false; aIm_hero_r[3, 2].Visible := false;
         aIm_box[ 9, 5].Visible := true;  aIm_hero_r[3, 3].Visible := true;

         Inc(iScore, iScoreMultiplier); Vid_Show_Panel_Score(iScore, true);

         Timer_Mario_Moved_Box.Interval := 200;
         Timer_Mario_Moved_Box.Enabled := true;
       end;
  end; // case..of

  // "chance time" starts, or game proceeds with beep
  if (iScore = 300) then
    Gam_Start_Chance_Time
  else
    Snd_Play(pfs_point);

  Timer_Mario_Moving_Box.Enabled := false;
end;





procedure TForm1.Timer_MissTimer(Sender: TObject);
{ a miss has occured (on Luigi's or Mario's side, depending on "iErrorCode")   }
begin
  case iLoopMiss of
    1: begin aIm_miss_code[iLoopMissCode].Visible := false;                      Timer_Miss.Interval := 200; end;
    2: begin aIm_miss_code[iLoopMissCode].Visible := true;  Snd_Play(pfs_miss1); end;
    3:       aIm_miss_code[iLoopMissCode].Visible := false;
    4: begin aIm_miss_code[iLoopMissCode].Visible := true;  Snd_Play(pfs_miss1); Timer_Miss.Interval := 100; end;
    5:       aIm_miss_code[iLoopMissCode].Visible := false;
    6: begin
         if iLoopMissCode = 1 then
           aIm_miss[2, 1].Visible := true    // 1
         else
           if Odd(iLoopMissCode) then
             aIm_miss[2, 2].Visible := true  // 3, 5
           else
             aIm_miss[1, 1].Visible := true; // 2, 4, 6

         Snd_Play(pfs_miss2);

         Timer_Miss.Interval := 450;
       end;
    7: begin
         if iLoopMissCode = 1 then
           aIm_miss[2, 1].Visible := false    // 1
         else
           if Odd(iLoopMissCode) then
             aIm_miss[2, 2].Visible := false  // 3, 5
           else
             aIm_miss[1, 1].Visible := false; // 2, 4, 6

         if Odd(iLoopMissCode) then
           begin Vid_Show_All_Mario(false); aIm_miss[2, 3].Visible := true; aIm_miss[2, 4].Visible := true; end
         else
           begin Vid_Show_All_Luigi(false); aIm_miss[1, 2].Visible := true; aIm_miss[1, 3].Visible := true; end;

         Snd_Play(pfs_miss1);

         Timer_Miss.Interval := 450;
       end;
    8: begin
         if Odd(iLoopMissCode) then
           begin aIm_miss[2, 4].Visible := false; aIm_miss[2, 5].Visible := true; end
         else
           begin aIm_miss[1, 3].Visible := false; aIm_miss[1, 4].Visible := true; end;

         Snd_Play(pfs_miss1);
       end;
    9: begin
         if Odd(iLoopMissCode) then
           begin aIm_miss[2, 5].Visible := false; aIm_miss[2, 4].Visible := true; end
         else
           begin aIm_miss[1, 4].Visible := false; aIm_miss[1, 3].Visible := true; end;

         Snd_Play(pfs_miss1);
       end;
   10: begin
         if Odd(iLoopMissCode) then
           begin aIm_miss[2, 4].Visible := false; aIm_miss[2, 5].Visible := true; end
         else
           begin aIm_miss[1, 3].Visible := false; aIm_miss[1, 4].Visible := true; end;

         Snd_Play(pfs_miss1);
       end;
   11: begin
         if Odd(iLoopMissCode) then
           begin aIm_miss[2, 5].Visible := false; aIm_miss[2, 4].Visible := true; end
         else
           begin aIm_miss[1, 4].Visible := false; aIm_miss[1, 3].Visible := true; end;

         Snd_Play(pfs_miss1);

         Timer_Miss.Interval := 250;
       end;
   12: begin
         Inc(iMiss); Vid_Show_Panel_Miss(iMiss);
         Snd_Play(pfs_miss2);

         Timer_Miss.Interval := 500;
       end;
   13: begin                                                                    // game reprises or is over
         if iMiss < 3 then
           begin                                                                // game reprises
             // removes boxes too close to the edge of conveyor belts
             aIm_box[ 0, 1].Visible := false; aIm_box[ 1, 1].Visible := false;
             aIm_box[ 0, 3].Visible := false; aIm_box[ 1, 3].Visible := false;
             aIm_box[ 0, 5].Visible := false; aIm_box[ 1, 5].Visible := false;
             aIm_box[ 8, 2].Visible := false; aIm_box[ 9, 2].Visible := false;
             aIm_box[ 8, 4].Visible := false; aIm_box[ 9, 4].Visible := false;
             aIm_box[10, 1].Visible := false; aIm_box[11, 1].Visible := false;

             // displays main characters "in game"
             Vid_Show_All_Miss(false);
             Vid_Show_Luigi(iPosLuigi, true);
             Vid_Show_Mario(iPosMario, true);

             // ends chance time, regardless of previous value
             iScoreMultiplier := 1;

             // game reprises
             Gam_Set_Timer_Game_Interval;
             Timer_Game.Enabled := true;

             if iLoopBoxToVan <> 0 then
               begin
                 Timer_Box_To_Van.Interval := 200;
                 Timer_Box_To_Van.Enabled := true;
               end;

              if iBoxesOnVan > 6 then
                begin
                  Timer_Van_Smoke.Interval := 150;
                  Timer_Van_Smoke.Enabled := true;
               end;

             bControllerLuigi := true;
             bControllerMario := true;
           end
         else
           begin                                                                // game is over
             if (iMode = 6) and (iScore > iHiScoreA) then
               begin
                 iHiScoreA := iScore;
                 iReg.WriteInteger('', 'HiScore1', iHiScoreA);
               end;

             if (iMode = 8) and (iScore > iHiScoreB) then
               begin
                 iHiScoreB := iScore;
                 iReg.WriteInteger('', 'HiScore2', iHiScoreB);
               end;

             iMode := 9; Gam_Set_Mode;

             iLoopGameOver := 1;
             Timer_Game_Over.Interval := 1000;
             Timer_Game_Over.Enabled := true;
           end;

         Timer_Miss.Enabled:=false;
       end;
  end; // case..of

  Inc(iLoopMiss);
end;






procedure TForm1.Timer_Move_Luigi_DownTimer(Sender: TObject);
{ enables controller, moves Luigi and disables timer                           }
begin
  bSWKeyPressed := false;
  Inp_2_sw_down;

  Timer_Move_Luigi_Down.Enabled := false;
end;





procedure TForm1.Timer_Move_Luigi_UpTimer(Sender: TObject);
{ enables controller, moves Luigi and disables timer                           }
begin
  bNWKeyPressed := false;
  Inp_1_nw_down;

  Timer_Move_Luigi_Up.Enabled := false;
end;





procedure TForm1.Timer_Move_Mario_DownTimer(Sender: TObject);
{ enables controller, moves Mario and disables timer                           }
begin
  bSEKeyPressed := false;
  Inp_4_se_down;

  Timer_Move_Mario_Down.Enabled := false;
end;





procedure TForm1.Timer_Move_Mario_UpTimer(Sender: TObject);
{ enables controller, moves Mario and disables timer                           }
begin
  bNEKeyPressed := false;
  Inp_3_ne_down;

  Timer_Move_Mario_Up.Enabled := false;
end;





procedure TForm1.Timer_Score_BlinkTimer(Sender: TObject);
{ score blinks during "chance-time"                                            }
begin
  Vid_Show_Panel_Score(iScore, not Odd(iLoopScoreBlink));

  Inc(iLoopScoreBlink);

  if iLoopScoreBlink = 3 then
    iLoopScoreBlink := 1;
end;





procedure TForm1.Timer_Van_SmokeTimer(Sender: TObject);
{ smoke animation (occurs when van is almost full, ready to leave)             }
begin
  if aIm_van[5].Visible and aIm_van[6].Visible then
    begin
      aIm_van[5].Visible := false;
      aIm_van[6].Visible := false;
    end
  else
    if aIm_van[5].Visible and (not aIm_van[6].Visible) then
      aIm_van[6].Visible := true
    else
      if (not aIm_van[5].Visible) and (not aIm_van[6].Visible) then
        aIm_van[5].Visible := true;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.pm_0_menuPopup(Sender: TObject);
{ shows the "checked" menuitems if needed. also, hides the game menu if it is  }
{ visible when the popup menu pops up                                          }
begin
{}pm_11_splashscreen.Checked := iReg.ReadBool('', 'SplashScreen', SplashForm.chb_splashscreen.Checked);
{}pm_12_controller.Checked := bDisplayCtrlOverlays;

{}if bFullscreenMode
    and bGameMenuOpen then
    begin
      bGameMenuOpen := false;
      Form1.Vid_Display_Game_Menu(bGameMenuOpen);
    end;
end;





procedure TForm1.pm_1_aboutClick(Sender: TObject);
{ displays the embedded "About the author..." form                             }
begin
  AboutForm.Show;
end;





procedure TForm1.pm_2_readmeClick(Sender: TObject);
{ opens the "Readme.txt" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Readme.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_3_whatsnewClick(Sender: TObject);
{ opens the "Whatsnew.txt" file                                                }
begin
  ShellExecute(Handle, 'open', 'Whatsnew.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_4_manualClick(Sender: TObject);
{ displays the embedded instructions manual                                    }
begin
  ManualForm.Show;
end;





procedure TForm1.pm_6_configClick(Sender: TObject);
{ opens the "Config.ini" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Config.ini', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_7_keysClick(Sender: TObject);
{ displays the "key settings" form                                             }
begin
  KeysForm.position := poScreenCenter;
  KeysForm.visible := true;
  Form1.Enabled := false;
end;





procedure TForm1.pm_8_hiscoreClick(Sender: TObject);
{ resets high scores to the system registry                                    }
begin
  iHiScoreA := 0; iReg.WriteInteger('', 'HiScore1', iHiScoreA);
  iHiScoreB := 0; iReg.WriteInteger('', 'HiScore2', iHiScoreB);
  Beep;
end;





{}procedure TForm1.pm_10_fullscreenClick(Sender: TObject);
{ changes from fullscreen to windowed, and vice-versa                          }
begin
  // updates popup menu
  bFullscreenMode := not bFullscreenMode;
  pm_10_fullscreen.Checked := bFullscreenMode;
  bGameMenuOpen := false;

  iReg.WriteBool('', 'FullscreenMode', bFullscreenMode);

  // graphics init
  Top  := - Screen.Height;
  Left := - Screen.Width;
  Gfx_Game_Init;
  Position := poScreenCenter;
end;





procedure TForm1.pm_11_splashscreenClick(Sender: TObject);
{ updates the ticks to the checkbox, the popup menu and the system registry    }
begin
{}pm_11_splashscreen.Checked := not pm_11_splashscreen.Checked;

{}iReg.WriteBool('', 'SplashScreen', pm_11_splashscreen.Checked);
end;





procedure TForm1.pm_12_controllerClick(Sender: TObject);
{ displays the "tranlucent" controller overlay on the directional cross        }
begin
{}pm_12_controller.Checked := not pm_12_controller.Checked;
{}bDisplayCtrlOverlays := pm_12_controller.Checked;

{}iReg.WriteBool('', 'CtrlrOverlay', pm_12_controller.Checked);

  Form1.Vid_Display_Ctrl_Overlay;
end;





procedure TForm1.pm_14_exitClick(Sender: TObject);
{ exits the game                                                               }
begin
  Application.Terminate;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.btn_game_menu_topClick(Sender: TObject);
{ sets the game menu as visible/hidden, then displays/hides it                 }
begin
  bGameMenuOpen := not bGameMenuOpen;
  Vid_Display_Game_Menu(bGameMenuOpen);
end;





procedure TForm1.la_menu_itemMouseEnter(Sender: TObject);
{ the game menu label bg colour becomes solid                                  }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := false;
end;





procedure TForm1.la_menu_itemMouseLeave(Sender: TObject);
{ the game menu label bg colour becomes transparent                            }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := true;
end;





procedure TForm1.la_menu_itemMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ hides the game menu, once a label has been pressed                           }
begin
  if Button = mbLeft then
    begin
      bGameMenuOpen := false;
      Vid_Display_Game_Menu(false);
    end;
end;





procedure TForm1.Vid_Display_Game_Menu(b: boolean);
{ displays/hides the game menu, depending on the value of bGameMenuOpen        }
  var
    i: integer;
begin
  case bGameMenuOpen of
    true: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i] := TLabel.Create(Form1);
            aMenuLabels[i].Parent := Form1;

            aMenuLabels[i].Top := 32 * i;
            aMenuLabels[i].Left := 0;

            aMenuLabels[i].AutoSize := false;
            aMenuLabels[i].Height := 32;
            aMenuLabels[i].Width := 256;
            aMenuLabels[i].Layout := tlCenter;

            aMenuLabels[i].Transparent := true;

            if aMenuItem[i].Caption <> '-' then
              begin
                aMenuLabels[i].Font.Name := 'Arial';
                aMenuLabels[i].Font.Color := clWhite;
                aMenuLabels[i].Font.Size := -12;
                aMenuLabels[i].Caption := '   ' + aMenuItem[i].Caption;

                if aMenuItem[i].Checked then
                  aMenuLabels[i].Caption := aMenuLabels[i].Caption + ' [V]';

                aMenuLabels[i].Color := $00000000;
                aMenuLabels[i].Cursor := crHandPoint;
                aMenuLabels[i].Enabled := aMenuItem[i].Enabled;

                if aMenuLabels[i].Enabled then
                  begin
                    aMenuLabels[i].OnClick      := aMenuItem[i].OnClick;
                    aMenuLabels[i].OnMouseEnter := la_menu_itemMouseEnter;
                    aMenuLabels[i].OnMouseLeave := la_menu_itemMouseLeave;
                    aMenuLabels[i].OnMouseUp    := la_menu_itemMouseUp;
                  end;
              end;
          end;

        btn_game_menu_background.Width := 256;
        btn_game_menu_background.Height := 32 * (MENU_ITEMS + 1)
      end;
    false: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i].Free;
          end;

        btn_game_menu_background.Height := 32;
        btn_game_menu_background.Width := 36;
      end;
    end; // case..of
end;





initialization
  bStartup := true;





end.
